/*
 * Copyright (c) 1996 Mark Salyzyn
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY MARK SALYZYN ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL MARK SALYZYN BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	@(#)hayes.h	1.8 (Salyzyn) 6/20/96
 */

#ifndef __HAYES_H__
#define __HAYES_H__ 1

/*
 *	Must wait for the corresponding bit to go active before accessing any
 * of the extended registers.
 */
#define hayes_ready		0
# define hr_rx1		0x01
# define hr_tx1		0x02
# define hr_rx2		0x04
# define hr_tx2		0x08
# define hr_stat1	0x10
# define hr_cmd1	0x20
# define hr_stat2	0x40
# define hr_cmd2	0x80
#define hayes_wait(w,x) while ((inb ((w) + hayes_ready) & (x)) != (x)) continue 
#define hayes_outb(w,x,y) hayes_wait(w,hr_##x); outb ((w) + hayes_##x, y)
#define hayes_inb(w,x,y) hayes_wait(w,hr_##x); *(y) = inb ((w) + hayes_##x)
/*
 *	Indicates the source of the interrupt.
 */
#define hayes_service_id	1
# define hs_rx1_fifo	0x01
# define hs_tx1_fifo	0x02
# define hs_error1	0x04
# define hs_dma_end	0x08
# define hs_rx2_fifo	0x10
# define hs_tx2_fifo	0x20
# define hs_error2	0x40
# define hs_dma_timout	0x80
/*
 *	Receiver and Transmitter registers
 */
#define	hayes_rx1		2
#define hayes_tx1		hayes_rx1
#define hayes_rx2		3
#define hayes_tx2		hayes_rx2
/*
 *	Command registers
 */
#define hayes_cmd1		4
#define hayes_stat1		hayes_cmd1
#define hayes_cmd2		5
#define hayes_stat2		hayes_cmd2
#define hayes_dma_rx		6
#define hayes_dma_tx		hayes_dma_rx
#define hayes_word_status	7

/*
 *	Hayes Extended commands
 */

/*
 *	The following commands are considered General Commands
 */
/*
 *	Flushes all FIFOs, zero all setup parameters, reset the ESP I/O
 * registers, restore the UART registers to their hardware reset state, and
 * then write 00 to hayes_stat1.
 */
#define HAYES_RESET		0x00
#define HAYES_RESET_1		HAYES_RESET
#define HAYES_RESET_2		HAYES_RESET
/*
 *	Get Firmware revision number in hayes_stat1, and self test results into
 * hayes_stat2.
 */
#define	HAYES_GET_SELF_TEST	0x01
#define HAYES_GET_SELF_TEST_1	HAYES_GET_SELF_TEST
#define HAYES_GET_SELF_TEST_2	HAYES_GET_SELF_TEST
#define ST_RAM_BAD	0x01
#define ST_ROM_BAD	0x02
#define ST_8K_ROM	0x04
#define ST_MCA_BUS	0x08
#define ST_HW_MASK	0xF0
#define ST_SINGLE_PORT	0x00
#define ST_DUAL_PORT	0x10
#define ST_V2		0x20 /* Hayes Communications Accelerator V2 returns */
/*
 *	Get DIP switch settings used for compatability mode.
 */
#define HAYES_GET_SWITCHES	0x02
#define HAYES_GET_SWITCHES_1	HAYES_GET_SWITCHES
#define HAYES_GET_SWITCHES_2	HAYES_GET_SWITCHES
#define GS_SW1_1	0x01
#define GS_SW1_2	0x02
#define GS_SW1_3	0x04
#define GS_SW1_3F8_IRQ4		0x00
#define GS_SW1_2F8_IRQ3		0x01
#define GS_SW1_3E8_IRQ4		0x02
#define GS_SW1_2E8_IRQ3		0x03
#define GS_SW1_1E0_IRQ9		0x04
#define GS_SW1_1E8_IRQ5		0x05
#define GS_SW1_280_IRQ9		0x06
#define GS_SW1_288_IRQ5		0x07
#define GS_SW2_1	(GS_SW1_1<<4)
#define GS_SW2_2	(GS_SW1_2<<4)
#define GS_SW2_3	(GS_SW1_3<<4)
#define GS_SW2_3F8_IRQ4		(GS_SW1_3F8_IRQ4<<4)
#define GS_SW2_2F8_IRQ3		(GS_SW1_2F8_IRQ3<<4)
#define GS_SW2_3E8_IRQ4		(GS_SW1_3E8_IRQ4<<4)
#define GS_SW2_2E8_IRQ3		(GS_SW1_2E8_IRQ3<<4)
#define GS_SW2_1E0_IRQ9		(GS_SW1_1E0_IRQ9<<4)
#define GS_SW2_1E8_IRQ5		(GS_SW1_1E8_IRQ5<<4)
#define GS_SW2_280_IRQ9		(GS_SW1_280_IRQ9<<4)
#define GS_SW2_288_IRQ5		(GS_SW1_288_IRQ5<<4)
/*
 *	Download (included her for completeness, do not use)
 */
#define HAYES_DOWNLOAD		0x03
#define HAYES_DOWNLOAD_1	HAYES_DOWNLOAD
#define HAYES_DOWNLOAD_2	HAYES_DOWNLOAD
typedef struct {
	unsigned char	address_msb ;
	unsigned char	address_lsb ;
	unsigned char	count ;		/* count + 1 */
	unsigned char	data[256] ;	/* size of download */
}	HAYES_DOWNLOAD_STRUCTURE ;	/* sent to hayes_cmd2 */

/*
 *	The following commands are considered Setup Commands.
 */
/*
 *	Set Enhanced mode IRQ and DRQ
 */
#define HAYES_SET_IRQ_DRQ	0x04
#define HAYES_SET_IRQ_DRQ_1	HAYES_SET_IRQ_DRQ
#define HAYES_SET_IRQ_DRQ_2	HAYES_SET_IRQ_DRQ
#define ID_IRQ_ENABLE	0x01
#define ID_IRQ9		0x00
#define ID_IRQ2		ID_IRQ9
#define ID_IRQ3		0x01
#define ID_IRQ4		0x02
#define ID_IRQ5		0x03
#define ID_DMA_1	0x10	/* 8 bit DMA */
#define ID_DMA_3	0x30	/* 16 bit DMA */
/*
 *	Set Enhanced IRQ
 */
#define HAYES_SET_IRQ	0x1F
#define HAYES_SET_IRQ_1	HAYES_SET_IRQ
#define HAYES_SET_IRQ_2	HAYES_SET_IRQ
#define I_SLAVE		13
/*
 *	Set the DMA timeout value. If a period of time equal to the timeout
 * occurs between two character transfers during a DMA, the transfer will halt
 * and issue a DMA timeout service request. Value is in tens of milliseconds,
 * recommended value is 100 (1 second).
 */
#define HAYES_SET_DMA_TIMEOUT	0x05
#define HAYES_SET_DMA_TIMEOUT_1	HAYES_SET_DMA_TIMEOUT
#define HAYES_SET_DMA_TIMEOUT_2	HAYES_SET_DMA_TIMEOUT
/*
 *	Set service request mask.
 */
#define HAYES_SET_SERVICE_MASK	0x06
#define HAYES_SET_SERVICE_MASK_1 HAYES_SET_SERVICE_MASK
#define HAYES_SET_SERVICE_MASK_2 HAYES_SET_SERVICE_MASK
#define SSM_RX_FIFO1_TRIGGER		0x01
#define SSM_TX_FIFO1_TRIGGER		0x02
#define SSM_ERROR1			0x04
#define SSM_RX_FIFO2_TRIGGER		(SSM_RX_FIFO1_TRIGGER<<4)
#define SSM_TX_FIFO2_TRIGGER		(SSM_TX_FIFO1_TRIGGER<<4)
#define SSM_ERROR2			(SSM_ERROR1<<4)
#define SSM_DMA_COMPLETE		0x08
#define SSM_DMA_TIMEOUT			0x80
/*
 *	Set error status mask.
 */
#define HAYES_SET_ERROR_MASK_1	0x07
#define HAYES_SET_ERROR_MASK_2	0x87
#define	SEM_RX_TIMEOUT			0x0001
#define SEM_RX_FIFO_OVERFLOW		0x0002
#define	SEM_PARITY			0x0004
#define	SEM_FRAMING			0x0008
#define	SEM_BREAK			0x0010
#define	SEM_UART			0x0020
#define	SEM_FLOW_OFF_TIMEOUT		0x0040
#define	SEM_FINISHED_BREAK		0x0080
#define	SEM_REMOTE_TX_FLOWED_OFF	0x0100
#define SEM_LOCAL_TX_FLOWED_OFF		0x0200
#define	SEM_LOCAL_TX_FLOWED_ON		0x0400
/*
 *	Set flow control type
 */
#define	HAYES_SET_FLOW_TYPE_1	0x08
#define	HAYES_SET_FLOW_TYPE_2	0x88
#define	DFT_RX_XON_XOFF			0x0001
#define DFT_TRANSPARENT_XON_XOFF	0x0002	/* Don't set other flow type */
#define DFT_RX_RTS			0x0004
#define DFT_RX_DTR			0x0008
#define DFT_SET_RTS			0x0010
#define DFT_SET_DTR			0x0020
#define DFT_SET_LOOP_BIT		0x0040	/* Diagnostics only */
#define DFT_TX_XON_XOFF			0x0080
#define DFT_TX_CTS			0x1000
#define DFT_TX_DSR			0x2000
/*
 *	Set Flow Control characters.
 */
#define HAYES_SET_FLOW_CHARS_1	0x09
#define HAYES_SET_FLOW_CHARS_2	0x89
typedef struct {
	unsigned char	xon ;	/* Typically 0x11 ^Q */
	unsigned char	xoff ;	/* Typically 0x13 ^S */
	unsigned char	escape ;/* Typically 0x10 ^P */
	unsigned char	xor ;	/* Typically 0x21 ! */
	unsigned char	mask ;	/* Parity mask, 0x7F for 7 bit match */
}	HAYES_SET_FLOW_CHARS_STRUCTURE ;	/* sent to hayes_cmd2 */
/*
 *	Set receive FIFO Flow Control level
 */
#define HAYES_SET_FLOW_THRESHOLD_1	0x0A
#define HAYES_SET_FLOW_THRESHOLD_2	0x8A
typedef struct {
	unsigned char	msb_hiwater ;	/* Typically 768 / 256 */
	unsigned char	lsb_hiwater ;	/* Typically 768 % 256 */
	unsigned char	msb_lowater ;	/* Typically 512 / 256 */
	unsigned char	lsb_lowater ;	/* Typically 512 % 256 */
}	HAYES_SET_FLOW_THRESHOLD_STRUCTURE ;	/* sent to hayes_cmd2 */
/*
 *	Set receive FIFO Trigger level
 */
#define HAYES_SET_TRIGGER_THRESHOLD_1	0x0B
#define HAYES_SET_TRIGGER_THRESHOLD_2	0x8B
typedef struct {
	unsigned char	msb_rx ;	/* Typically 256 / 256 */
	unsigned char	lsb_rx ;	/* Typically 256 % 256 */
	unsigned char	msb_tx ;	/* Typically 768 / 256 */
	unsigned char	lsb_tx ;	/* Typically 768 % 256 */
}	HAYES_SET_TRIGGER_THRESHOLD_STRUCTURE ;	/* sent to hayes_cmd2 */
/*
 *	Set receive character character timeout in milliseconds. It is
 * suggested to set this value to four character times to wait before
 * creating a receive interrupt after this amount of dead time. It is possible
 * to get an interrupt from this condition even if the buffer is empty because
 * of the race condition created by the delay.
 */
#define HAYES_SET_RX_TIMEOUT_1	0x0C
#define HAYES_SET_RX_TIMEOUT_2	0x8C
#define SRT_DISABLE	0
/*
 *	Set flowed off timeout. This value in seconds is used to issue an
 * interrupt for a locked up flow control. A typical value for this parameter
 * is 60 seconds.
 */
#define HAYES_SET_FLOW_TIMEOUT_1	0x0D
#define HAYES_SET_FLOW_TIMEOUT_2	0x8D
#define SFT_DISABLE	0
/*
 *	Write to UART. Provides a means of talking to the `16550' compatable
 * port set even if the communication channel does not have the accompanying
 * compatibility registers accessible via the I/O map.
 */
#define HAYES_WRITE_UART_1	0x0E
#define HAYES_WRITE_UART_2	0x8E
typedef struct {
	unsigned char	address ;	/* value from 0 to 7 */
	unsigned char	value ;		/* value to write to the port */
}	HAYES_WRITE_UART_STRUCTURE ;	/* sent to hayes_cmd2 */
/*
 *	Read to UART. Provides a means of talking to the `16550' compatable
 * port set even if the communication channel does not have the accompanying
 * compatibility registers accessible via the I/O map. Address is sent via the
 * hayes_cmd2 register and read back in the hayes_stat1 register.
 */
#define HAYES_READ_UART_1	0x0F
#define HAYES_READ_UART_2	0x8F
/*
 *	Set Divisor Latch. Do not set this value to 1 (115200bps) in
 * enhanced mode.
 */
#define HAYES_SET_DIVISOR_1	0x1D
#define HAYES_SET_DIVISOR_2	0x9D
typedef struct {
	unsigned char	msb_value ;	/* (115200 / <baud rate>) / 256 */
	unsigned char	lsb_value ;	/* (115200 / <baud rate>) % 256 */
}	HAYES_SET_DIVISOR_STRUCTURE ;	/* sent to hayes_cmd2 */
/*
 *	Get Mode. The default mode is read in hayes_stat1, and th current
 * mode is read in hayes_stat2.
 */
#define HAYES_GET_MODE_1	0x1E
#define HAYES_GET_MODE_2	0x9E
#define GM_ENHANCED			0x01	/* Always 0 for default */
#define GM_FIFO_ENABLE			0x02
#define GM_RTS_FLOW_ENABLE		0x04
#define GM_DTR_FLOW_ENABLE		0x08
#define GM_RX_DMA			0x10	/* Always 0 for default */
#define GM_TX_DMA			0x20	/* Always 0 for default */
#define GM_COMPATIBILITY		0x80
/*
 *	Set Mode.
 */
#define HAYES_SET_MODE_1	0x10
#define HAYES_SET_MODE_2	0x90
#define SM_COMPATIBILITY	GM_COMPATIBILITY
#define SM_ENHANCED		GM_ENHANCED
#define SM_FIFO_ENABLE		GM_FIFO_ENABLE
#define SM_RTS_FLOW_ENABLE	GM_RTS_FLOW_ENABLE
#define SM_DTR_FLOW_ENABLE	GM_DTR_FLOW_ENABLE
#define SM_RX_DMA		GM_RX_DMA
#define SM_TX_DMA		GM_TX_DMA
/*
 *	Set re-interrupt pacing timeout.
 */
#define HAYES_SET_PACING_TIMOUT		0x20
/*
 *	Get Service ID Mask.
 */
#define HAYES_GET_SERVICE_ID_MASK	0x21
/*
 *	Get Receive Error Count
 */
#define	HAYES_GET_RX_ERROR_COUNT	0x22
/*
 *	Set UART Clock Prescaler
 */
#define HAYES_SET_CLOCK_PRESCALER_1	0x23
#define HAYES_SET_CLOCK_PRESCALER_2	0xA3
#define HAYES_PRESCALER_1	0
#define HAYES_PRESCALER_2	1
#define HAYES_PRESCALER_4	2
#define HAYES_PRESCALER_8	3

/*
 *	The following commands are considered operational commands.
 */
/*
 *	Clear DMA Terminal Count and Timeout Service request. Do not use.
 */
#define HAYES_CLEAR_DMA		0x11
/*
 *	Get Error status.
 */
#define HAYES_GET_ERROR_1	0x12
#define HAYES_GET_ERROR_2	0x92
#define	GEM_RX_TIMEOUT			SEM_RX_TIMEOUT
#define GEM_RX_FIFO_OVERFLOW		SEM_RX_FIFO_OVERFLOW
#define	GEM_PARITY			SEM_PARITY
#define	GEM_FRAMING			SEM_FRAMING
#define	GEM_BREAK			SEM_BREAK
#define	GEM_UART			SEM_UART
#define	GEM_FLOW_OFF_TIMEOUT		SEM_FLOW_OFF_TIMEOUT
#define	GEM_FINISHED_BREAK		SEM_FINISHED_BREAK
#define	GEM_REMOTE_TX_FLOWED_OFF	SEM_REMOTE_TX_FLOWED_OFF
#define	GEM_LOCAL_TX_FLOWED_OFF		SEM_LOCAL_TX_FLOWED_OFF
#define	GEM_LOCAL_TX_FLOWED_ON		SEM_LOCAL_TX_FLOWED_ON
/* 
 *	Get UART Status, hayes_stat1 contains the LSR, the hayes_stat2
 * contains the MSR. Use this command in Enhanced Mode only, because this
 * access in compatability mode will clear certain bits the driver will need.
 * The MSR delta bits are not to be trusted.
 */
#define HAYES_GET_UART_STATUS_1	0x13
#define HAYES_GET_UART_STATUS_2	0x93
/*
 *	Get receive bytes available. hayes_stat1 will contain the MSB,
 * hayes_stat2 will contain the LSB. Does not include the count of bytes in
 * the UART receive register.
 */
#define HAYES_GET_RX_COUNT_1	0x14
#define HAYES_GET_RX_COUNT_2	0x94
/*
 *	Get transmit space available. hayes_stat1 will contain the MSB,
 * hayes_stat2 will contain the LSB.
 */
#define HAYES_GET_TX_COUNT_1	0x15
#define HAYES_GET_TX_COUNT_2	0x95
/*
 *	Initiate DMA receive.
 */
#define	HAYES_DMA_RX_1		0x16
#define	HAYES_DMA_RX_2		0x96
/*
 *	Initiate DMA transmit.
 */
#define	HAYES_DMA_TX_1		0x17
#define	HAYES_DMA_TX_2		0x97
/*
 *	Flow Control Off transmitter.
 */
#define HAYES_TX_OFF_1		0x18
#define HAYES_TX_OFF_2		0x98
/*
 *	Flow Control On transmitter.
 */
#define HAYES_TX_ON_1		0x19
#define HAYES_TX_ON_2		0x99
/*
 *	Line Break. Waits for all characters to flush from both the FIFO and
 * the UART transmitter, and then issues a break in the duration specified in
 * the hayes_cmd2 register in tens of milliseconds.
 */
#define HAYES_BREAK_1		0x1A
#define HAYES_BREAK_2		0x9A
/*
 *	Flush receiver.
 */
#define HAYES_RX_FLUSH_1	0x1B
#define HAYES_RX_FLUSH_2	0x9B
/*
 *	Flush transmitter.
 */
#define HAYES_TX_FLUSH_1	0x1C
#define HAYES_TX_FLUSH_2	0x9C

#endif	/* __HAYES_H__ */
